InternalSubGraph

Struct InternalSubGraph 

pub struct InternalSubGraph {
    pub filter: SuBitGraph,
    pub loopcount: Option<usize>,
}
Represents a subgraph consisting entirely of “internal” edges.

An internal subgraph is defined such that if a half-edge is part of this subgraph, its opposite half-edge must also be part of the subgraph. This means it cannot contain any “dangling” or “external” half-edges that connect to parts of the graph outside of itself. It represents a self-contained portion of the graph’s edge topology.

To represent a subgraph that does have external connections (or “hairs”), use types like HedgeNode or [ContractedSubGraph].

Fields

filter: SuBitGraph

A bitmask representing the set of half-edges included in this internal subgraph. For every half-edge h included in this filter, its opposite inv(h) must also be included.

loopcount: Option<usize>

An optional field, often used to cache the cyclomatic number (number of independent cycles) within this subgraph once computed.

Implementations

Source

impl InternalSubGraph

pub unsafe fn new_unchecked(filter: SuBitGraph) -> Self

Create a new subgraph from a filter.

Safety

The filter must be valid, i.e. it must always have paired hedges.

Source

impl InternalSubGraph

pub fn add_edge<E, V, H, N: NodeStorageOps<NodeData = V>>( &mut self, hedge: Hedge, graph: &HedgeGraph<E, V, H, N>, )

pub fn remove_edge<E, V, H, N: NodeStorageOps<NodeData = V>>( &mut self, hedge: Hedge, graph: &HedgeGraph<E, V, H, N>, )

pub fn try_new<E, V, H, N: NodeStorageOps<NodeData = V>>( filter: SuBitGraph, graph: &HedgeGraph<E, V, H, N>, ) -> Option<Self>

pub fn cleaned_filter_optimist<E, V, H, N: NodeStorageOps<NodeData = V>>( filter: SuBitGraph, graph: &HedgeGraph<E, V, H, N>, ) -> Self

pub fn cleaned_filter_pessimist<E, V, H, N: NodeStorageOps<NodeData = V>>( filter: SuBitGraph, graph: &HedgeGraph<E, V, H, N>, ) -> Self

pub fn valid<E, V, H, N: NodeStorageOps<NodeData = V>>( &self, graph: &HedgeGraph<E, V, H, N>, ) -> bool

pub fn to_hairy_subgraph<E, V, H, N: NodeStorageOps<NodeData = V>>( &self, graph: &HedgeGraph<E, V, H, N>, ) -> HedgeNode

pub fn set_loopcount<E, V, H, N: NodeStorageOps<NodeData = V>>( &mut self, graph: &HedgeGraph<E, V, H, N>, )

pub fn cycle_basis<E, V, H, N: NodeStorageOps<NodeData = V>>( &self, graph: &HedgeGraph<E, V, H, N>, ) -> (Vec<Cycle>, SimpleTraversalTree)